@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2015
+/* Copyright (c) 1997-2018
    Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
    http://www.polymake.org
 
@@ -20,15 +20,19 @@
 #include "polymake/Array.h"
 #include "polymake/Set.h"
 #include "polymake/PowerSet.h"
-#include "polymake/graph/HasseDiagram.h"
+#include "polymake/graph/Lattice.h"
+#include "polymake/graph/Decoration.h"
 
 namespace polymake { namespace matroid {
 
+   using graph::Lattice;
+   using graph::lattice::Sequential;
+   using graph::lattice::BasicDecoration;
 
-Array<Set<int> > bases_from_lof_impl(const graph::HasseDiagram& LF, int n)
+Array<Set<int> > bases_from_lof_impl(const Lattice<BasicDecoration, Sequential>& LF, int n)
 {
-	int LF_dim = LF.dim();
-	if(LF_dim == -1) {
+	int LF_dim = LF.rank();
+	if(LF_dim == 0) {
 		//THis means the rank is 0
 		return Array<Set<int> >(1);
 	}
@@ -38,7 +42,7 @@ Array<Set<int> > bases_from_lof_impl(const graph::HasseDiagram& LF, int n)
       const Set<int> basis(*bit);
       bool dependent(false);
       for (int k=rank-1; k>=0; --k) {
-         for (Entire<graph::HasseDiagram::nodes_of_dim_set>::iterator fi=entire(LF.node_range_of_dim(k)); !dependent && !fi.at_end(); ++fi) {
+         for (auto fi=entire(LF.nodes_of_rank(k)); !dependent && !fi.at_end(); ++fi) {
             if (incl(basis, LF.face(*fi)) <= 0)
                dependent = true;
          }
