@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2015
+/* Copyright (c) 1997-2018
    Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
    http://www.polymake.org
 
@@ -23,11 +23,11 @@ namespace polymake { namespace matroid {
 
 void bases_from_lof(perl::Object M)
 {
-   const graph::HasseDiagram LF = M.give("LATTICE_OF_FLATS");
+   const perl::Object LF_obj = M.give("LATTICE_OF_FLATS");
+   const Lattice<BasicDecoration, Sequential> LF(LF_obj);
    const int n = M.give("N_ELEMENTS");
    const Array<Set<int> > bases = bases_from_lof_impl(LF, n);
-	int LF_dim = LF.dim();
-	if(LF_dim == -1) LF_dim = 0;
+	int LF_dim = LF.rank();
    M.take("RANK") << LF_dim; //Hasse diagram has a shift for lattice with just one node
    M.take("BASES") << bases;
    M.take("N_BASES") << bases.size();
