@@ -27,7 +27,7 @@
 #include "polymake/linalg.h"
 #include "polymake/PowerSet.h"
 #include "polymake/IncidenceMatrix.h"
-#include "polymake/tropical/LoggingPrinter.h"
+#include "polymake/common/lattice_tools.h"
 #include "polymake/tropical/thomog.h"
 #include "polymake/tropical/linear_algebra_tools.h"
 #include "polymake/tropical/lattice.h"
@@ -48,7 +48,7 @@ namespace polymake { namespace tropical {
 	 * */
 	Matrix<Integer> positive_decomposition(const Matrix<Rational> &rank_one_flats, const Matrix<Rational> &curve_rays) {
 		Matrix<Integer> result(curve_rays.rows(), rank_one_flats.rows());
-		//Iterate curve rays 
+		//Iterate curve rays
 		for(int cr = 0; cr < curve_rays.rows(); cr++) {
 			//Compute a linear representation of the vector in the rays
 			Vector<Rational> linRep = linearRepresentation(curve_rays.row(cr), rank_one_flats);
@@ -63,8 +63,8 @@ namespace polymake { namespace tropical {
 
 	/*
 	 * @brief Takes a list of rays of a curve given by their positive linear representations wrt to a rank-1-flat-vectors matrix and a list of weights of those rays. It then computes the degree of that curve.
-	 * @param Matrix<Rational> curve_decompositions 
-	 * @param Vector<Integer> curve_weights 
+	 * @param Matrix<Rational> curve_decompositions
+	 * @param Vector<Integer> curve_weights
 	 * @return Integer
 	 */
 	Integer degree_via_decomposition(const Matrix<Integer> &curve_decompositions, const Vector<Integer> &curve_weights) {
@@ -90,24 +90,24 @@ namespace polymake { namespace tropical {
 																 Matrix<Rational> &curve_b_rays,
 																 const Vector<Integer> &curve_b_weights) {
 		//Make everything integer
-		rank_one_flats = Matrix<Rational>(makePrimitiveInteger(rank_one_flats));
-		curve_a_rays = Matrix<Rational>(makePrimitiveInteger(curve_a_rays));
-		curve_b_rays = Matrix<Rational>(makePrimitiveInteger(curve_b_rays));
+		rank_one_flats = Matrix<Rational>(common::primitive(rank_one_flats));
+		curve_a_rays = Matrix<Rational>(common::primitive(curve_a_rays));
+		curve_b_rays = Matrix<Rational>(common::primitive(curve_b_rays));
 
-		Matrix<Integer> curve_a_decompositions = positive_decomposition( rank_one_flats, curve_a_rays); 
+		Matrix<Integer> curve_a_decompositions = positive_decomposition( rank_one_flats, curve_a_rays);
 		Matrix<Integer> curve_b_decompositions = positive_decomposition( rank_one_flats, curve_b_rays);
 
 		Integer result = degree_via_decomposition( curve_a_decompositions, curve_a_weights) *
 								degree_via_decomposition( curve_b_decompositions, curve_b_weights);
 
 
-		//We iterate pairs of rays of the two curves 
+		//We iterate pairs of rays of the two curves
 		for(int aray = 0; aray < curve_a_rays.rows(); aray++) {
 			Vector<Integer> amap = curve_a_decompositions.row(aray);
 			for(int bray = 0; bray < curve_b_rays.rows(); bray++) {
 				Vector<Integer> bmap = curve_b_decompositions.row(bray);
 				Integer correction = 0;
-				//Iterate pairs of rank one flats 
+				//Iterate pairs of rank one flats
 				for(pm::Subsets_of_k_iterator<const pm::Series<int,true> &> pair = entire(all_subsets_of_k( sequence(0,rank_one_flats.rows()),2)); !pair.at_end(); pair++) {
 					Vector<int> pair_as_vector(*pair);
 					correction = std::max( correction,
@@ -121,51 +121,52 @@ namespace polymake { namespace tropical {
 			}//END iterate curve B rays
 		}//END iterate curve A rays
 
-		
+
 
 		return result;
 	}//END intersection_multiplicity_in_uniform
 
 	/*
 	 *	@brief Takes a smooth surface fan and finds vectors corresponding to the rank one flats in some matroidal
-	 *	realization 
+	 *	realization
 	 * @param perl::Object surface A tropical surface fan.
 	 *	@return Matrix<Rational> The rank one rays in non-tropically homogeneous coordinates.
 	 */
 	template <typename Addition>
-		Matrix<Rational> find_rank_one_vectors(perl::Object surface) {
-			perl::ListResult result = ListCallPolymakeFunction("is_smooth",surface);
-			//Sanity check 
-			if(((int)result[0]) == 0) throw std::runtime_error("Finding rank one vectors: Surface is not smooth.");
-			
-			//Extract matroid and map 
-			perl::Object matroid = result[1];
-			perl::Object face_lattice = matroid.give("LATTICE_OF_FLATS");
-			perl::Object map = result[2];
+        Matrix<Rational> find_rank_one_vectors(perl::Object surface)
+        {
+          perl::Object matroid, map;
+          bool is_smooth;
+          call_function("is_smooth", surface) >> is_smooth >> matroid >> map;
+          //Sanity check
+          if (!is_smooth) throw std::runtime_error("Finding rank one vectors: Surface is not smooth.");
 
-			//Extract rank one flats 
-			int N = matroid.give("N_ELEMENTS");
-			NodeMap<Directed,Set<int> > all_faces = face_lattice.give("FACES");
-			Set<int> rank_one_flats = face_lattice.CallPolymakeMethod("nodes_of_dim",1);
-			Matrix<Rational> map_matrix = map.give("MATRIX");
-				map_matrix = inv(map_matrix.minor(~scalar2set(0), ~scalar2set(0)));
-				map_matrix = thomog_morphism(map_matrix, zero_vector<Rational>(map_matrix.cols())).first;
+          perl::Object face_lattice = matroid.give("LATTICE_OF_FLATS");
 
-			Matrix<Rational> converted_vectors(0, map_matrix.cols());
+          //Extract rank one flats
+          int N = matroid.give("N_ELEMENTS");
+          NodeMap<Directed,Set<int>> all_faces = face_lattice.give("FACES");
+          Set<int> rank_one_flats = face_lattice.call_method("nodes_of_rank", 1);
 
-			for(Entire<Set<int> >::iterator j = entire(rank_one_flats); !j.at_end(); j++) {
-				Vector<Rational> fvector(N);
-				Set<int> flat = all_faces[*j];
-				fvector.slice(flat) = Addition::orientation() * ones_vector<Rational>(flat.size());
-				converted_vectors /= (map_matrix * fvector);
-			}
+          Matrix<Rational> map_matrix = map.give("MATRIX");
+          map_matrix = inv(map_matrix.minor(~scalar2set(0), ~scalar2set(0)));
+          map_matrix = thomog_morphism(map_matrix, zero_vector<Rational>(map_matrix.cols())).first;
 
-			return tdehomog(converted_vectors,0,0);
-		}//END find_rank_one_vectors
+          Matrix<Rational> converted_vectors(0, map_matrix.cols());
+
+          for (auto j = entire(rank_one_flats); !j.at_end(); ++j) {
+            Vector<Rational> fvector(N);
+            const Set<int>& flat = all_faces[*j];
+            fvector.slice(flat) = Addition::orientation() * ones_vector<Rational>(flat.size());
+            converted_vectors /= (map_matrix * fvector);
+          }
+
+          return tdehomog(converted_vectors, 0, 0);
+        }//END find_rank_one_vectors
 
 	/*
-	 * @brief Finds all the maximal cones of a surface containing a given point 
-	 * @param Vector<Rational> point, in tropical homogeneous coordinates with leading coordinate. 
+	 * @brief Finds all the maximal cones of a surface containing a given point
+	 * @param Vector<Rational> point, in tropical homogeneous coordinates with leading coordinate.
 	 * @param Matrix<Rational> facet_normals The [[FACET_NORMALS]]
 	 * @param Matrix<Rational> affine_normals The [[AFFINE_HULL_NORMALS]]
 	 * @param SparseMatrix<int> maximal_facets The [[MAXIMAL_POLYTOPES_FACETS]]
@@ -176,20 +177,19 @@ namespace polymake { namespace tropical {
 	 * @return perl::Object The star of the surface as a Cycle
 	 */
 	template <typename Addition>
-		perl::Object compute_surface_star(const Vector<Rational> &point, 
-				const Matrix<Rational> &facet_normals, 
-				const Matrix<Rational> &affine_normals, 
-				const SparseMatrix<int> &maximal_facets, 
+		perl::Object compute_surface_star(const Vector<Rational> &point,
+				const Matrix<Rational> &facet_normals,
+				const Matrix<Rational> &affine_normals,
+				const SparseMatrix<int> &maximal_facets,
 				const IncidenceMatrix<> &maximal_affine,
 				const Matrix<Rational> &vertices,
 				const Matrix<Rational> &lineality,
 				const IncidenceMatrix<> &cones) {
-			
+
 			//First of all find the set of all maximal cones containing the point.
 			Set<int> containing_cones;
-			for(int mc = 0; mc < cones.rows(); mc++) {
-				if( affine_normals.minor( maximal_affine.row(mc), All) * point != 
-						zero_vector<Rational>(maximal_affine.row(mc).size())) {
+			for (int mc = 0; mc < cones.rows(); mc++) {
+                                if (!is_zero(affine_normals.minor( maximal_affine.row(mc), All) * point)) {
 					continue;
 				}
 				bool found_non_facet = false;
@@ -201,18 +201,18 @@ namespace polymake { namespace tropical {
 					}
 				}
 				if(found_non_facet) continue;
-				//Arriving here, it must contain the point 
+				//Arriving here, it must contain the point
 				containing_cones += mc;
 			}//END iterate maximal cones
 
-			//Now construct the star itself 
+			//Now construct the star itself
 			Matrix<Rational> star_rays = vertices;
-			Matrix<Rational> star_lineality = lineality; 
+			Matrix<Rational> star_lineality = lineality;
 			IncidenceMatrix<> star_cones = cones;
 
 			std::pair<Set<int>, Set<int> > f_and_nf = far_and_nonfar_vertices(vertices);
 			Set<int> cone_intersection = accumulate( rows( cones.minor(containing_cones,All)), operations::mul());
-			Set<int> unused_rays = sequence(0, vertices.rows()) - 
+			Set<int> unused_rays = sequence(0, vertices.rows()) -
 				accumulate( rows(cones.minor(containing_cones,All)),operations::add());
 
 			//Start by constructing the lineality space
@@ -233,9 +233,9 @@ namespace polymake { namespace tropical {
 				star_rays.row(*nfr) = star_rays.row(*nfr) - star_rays.row( intersect_nonfar[0]);
 			}
 
-			//Make the reference vertex the origin 
+			//Make the reference vertex the origin
 			star_rays.row( intersect_nonfar[0]) = unit_vector<Rational>( star_rays.cols(),0);
-		
+
 			perl::Object result(perl::ObjectType::construct<Addition>("Cycle"));
 				result.take("VERTICES") << star_rays.minor(~rays_to_remove,All);
 				result.take("MAXIMAL_POLYTOPES") << star_cones.minor(containing_cones, ~rays_to_remove);
@@ -248,17 +248,17 @@ namespace polymake { namespace tropical {
 
 	/*
 	 * @brief Computes the rays of the star of a curve at a point
-	 * @param Matrix<Rational> curve_vertices The curve's vertices (non-homog. with leading coord) 
-	 * @param IncidenceMatrix<> curve_cones The cuve's cones 
+	 * @param Matrix<Rational> curve_vertices The curve's vertices (non-homog. with leading coord)
+	 * @param IncidenceMatrix<> curve_cones The cuve's cones
 	 * @param IncidenceMatrix<> curve_containers The indices of all the maximal cones containing the point.
 	 * @param Matrix<Rationa> lineality If the curve consists only of a lineality space, this is its generator.
 	 * @return Matrix<Rational> The star rays, non-homog and without(!) leading coordinate.
 	 */
 	std::pair<Matrix<Rational>, Vector<Integer> > compute_curve_star_rays(
-			const Matrix<Rational> &curve_vertices, const IncidenceMatrix<> &curve_cones, 
+			const Matrix<Rational> &curve_vertices, const IncidenceMatrix<> &curve_cones,
 			const Vector<Integer> &curve_weights, const Set<int> &curve_containers, const Matrix<Rational> &lineality ) {
-		//Split up the rays in far and nonfar 
-		std::pair<Set<int>, Set<int> > f_and_nf = far_and_nonfar_vertices(curve_vertices);	
+		//Split up the rays in far and nonfar
+		std::pair<Set<int>, Set<int> > f_and_nf = far_and_nonfar_vertices(curve_vertices);
 
 		Matrix<Rational> result(0,curve_vertices.cols()-1);
 		Vector<Integer> star_weights;
@@ -272,15 +272,15 @@ namespace polymake { namespace tropical {
 			return std::make_pair(result, star_weights);
 		}
 
-		//If there is only one maximal cone, we compute its direction vector 
+		//If there is only one maximal cone, we compute its direction vector
 		if(curve_containers.size() == 1) {
 			Set<int> single_cone = curve_cones.row( *(curve_containers.begin()));
 			Set<int> far_in_container = f_and_nf.first * single_cone;
 			Vector<Rational> direction;
 			if(far_in_container.size() == 0) {
-				//Its a bounded edge 
+				//Its a bounded edge
 				Vector<int> both_rays(single_cone);
-				direction = 
+				direction =
 					curve_vertices.row( both_rays[0]) - curve_vertices.row(both_rays[1]);
 			}
 			else {
@@ -294,7 +294,7 @@ namespace polymake { namespace tropical {
 			return std::make_pair(result, star_weights);
 		}
 		else {
-			//If its several, then every one of them must have point as a vertex 
+			//If its several, then every one of them must have point as a vertex
 			int points_index = *( accumulate(rows(curve_cones.minor(curve_containers,All)), operations::mul()).begin());
 			for(Entire<Set<int> >::const_iterator adjRays = entire(curve_containers); !adjRays.at_end(); adjRays++) {
 				Vector<Rational> direction;
@@ -314,17 +314,17 @@ namespace polymake { namespace tropical {
 	}//END compute_curve_star_rays
 
 	/*
-	 * @brief For a curve and a point, computes the indices of all maximal cones containing the point 
-	 * @param Vector<Rational> point the point, non-homog, with leading coord 
-	 * @param Matrix<Rational> curve_rays The curve vertices, non-homog, with leading coord 
-	 * @param IncidenceMatrix curve_cones The curve cones 
-	 * @param int some_container An index of one maximal cone containing the point 
-	 * @return Set<int> All indices of cones containing the point 
+	 * @brief For a curve and a point, computes the indices of all maximal cones containing the point
+	 * @param Vector<Rational> point the point, non-homog, with leading coord
+	 * @param Matrix<Rational> curve_rays The curve vertices, non-homog, with leading coord
+	 * @param IncidenceMatrix curve_cones The curve cones
+	 * @param int some_container An index of one maximal cone containing the point
+	 * @return Set<int> All indices of cones containing the point
 	 */
 	Set<int> compute_containing_cones(const Vector<Rational> &point, const Matrix<Rational> &curve_rays,
 				const IncidenceMatrix<> &curve_cones, int some_container) {
-		//Go through the rays of the container - if point is equal to one of them, return all 
-		//maximal cones using the vertex 
+		//Go through the rays of the container - if point is equal to one of them, return all
+		//maximal cones using the vertex
 		Set<int> container_set = curve_cones.row(some_container);
 		for(Entire<Set<int> >::iterator cray = entire(container_set); !cray.at_end(); cray++) {
 			if(point == curve_rays.row(*cray)) {
@@ -332,37 +332,37 @@ namespace polymake { namespace tropical {
 			}
 		}
 
-		//If it isn't, it's just the one cone 
+		//If it isn't, it's just the one cone
 		return scalar2set(some_container);
 	}
 
 
 	template <typename Addition>
 		perl::Object intersect_in_smooth_surface(perl::Object surface, perl::Object cycle_a, perl::Object cycle_b) {
-			//Extract data 
+			//Extract data
 			int dim_a = cycle_a.give("PROJECTIVE_DIM");
 			int dim_b = cycle_b.give("PROJECTIVE_DIM");
 			int ambient_dim = surface.give("PROJECTIVE_AMBIENT_DIM");
 
-			//Basic sanity checks 
+			//Basic sanity checks
 			if(dim_a + dim_b <= 1) return empty_cycle<Addition>(ambient_dim);
 			if(dim_a > 2 || dim_b > 2) throw std::runtime_error("intersect_in_smooth_surface: Cycles dimension too large.");
 
-			//If one is full-dimensional, return the other one with multiplied weights 
+			//If one is full-dimensional, return the other one with multiplied weights
 			Vector<Integer> weights_a = cycle_a.give("WEIGHTS");
 			Vector<Integer> weights_b = cycle_b.give("WEIGHTS");
-			if(dim_a == 2) {
-				return cycle_b.CallPolymakeMethod("multiply_weights",weights_a[0]);
+			if (dim_a == 2) {
+				return cycle_b.call_method("multiply_weights", weights_a[0]);
 			}
-			if(dim_b == 2) {
-				return cycle_a.CallPolymakeMethod("multiply_weights", weights_b[1]);
+			if (dim_b == 2) {
+				return cycle_a.call_method("multiply_weights", weights_b[1]);
 			}
-			
+
 			//From here on we know we have two curves.
-			
-			//Refine both curves along the surface 
+
+			//Refine both curves along the surface
 			RefinementResult refined_cycle_a = refinement( cycle_a, surface, false, false, false, true, false);
-			RefinementResult refined_cycle_b = refinement( cycle_b, surface, false, false, false, true, false); 
+			RefinementResult refined_cycle_b = refinement( cycle_b, surface, false, false, false, true, false);
 			Matrix<Rational> rays_a = refined_cycle_a.complex.give("VERTICES");
 				rays_a = tdehomog(rays_a);
 			Matrix<Rational> rays_b = refined_cycle_b.complex.give("VERTICES");
@@ -381,7 +381,7 @@ namespace polymake { namespace tropical {
 					rays_a, lin_a, cones_a, rays_b, lin_b, cones_b);
 
 
-			//The potential intersection points are the nonfar vertices of this 
+			//The potential intersection points are the nonfar vertices of this
 			Matrix<Rational> ab_vertices = ab_intersect.rays;
 			Matrix<Rational> ab_vertices_homog = thomog(ab_vertices);
 			IncidenceMatrix<> ab_vertices_by_cones = T(ab_intersect.cones);
@@ -399,17 +399,17 @@ namespace polymake { namespace tropical {
 			for(int point = 0; point < ab_vertices.rows(); point++) {
 				//If it's a ray, ignore it.
 				if(ab_vertices(point,0) == 0) {
-					continue;		
+					continue;
 				}
 
 				//If it's a vertex, we need to compute the star of X at that point.
 				perl::Object surface_star = compute_surface_star<Addition>(
-						ab_vertices_homog.row(point), 
-						facet_normals, 
-						affine_hull, 
-						maximal_facets, 
-						maximal_affine, 
-						surface_rays, 
+						ab_vertices_homog.row(point),
+						facet_normals,
+						affine_hull,
+						maximal_facets,
+						maximal_affine,
+                                                surface_rays,
 						surface_lineality,
 						surface_cones);
 				Matrix<Rational> rank_one_vectors = find_rank_one_vectors<Addition>(surface_star);
@@ -419,15 +419,15 @@ namespace polymake { namespace tropical {
 				int cone_of_a = *(ab_intersect.xcontainers.row(cone_of_point).begin());
 				int cone_of_b = *(ab_intersect.ycontainers.row(cone_of_point).begin());
 				std::pair<Matrix<Rational>, Vector<Integer> > a_star =
-					compute_curve_star_rays(rays_a, cones_a, refweights_a, 
+					compute_curve_star_rays(rays_a, cones_a, refweights_a,
 						compute_containing_cones( ab_vertices.row(point), rays_a, cones_a, cone_of_a), lin_a);
 				std::pair<Matrix<Rational>, Vector<Integer> > b_star =
 					compute_curve_star_rays(rays_b, cones_b, refweights_b,
 						compute_containing_cones( ab_vertices.row(point), rays_b, cones_b, cone_of_b), lin_b);
 
 
-				//Finally, we can compute the multiplicity 
-				
+				//Finally, we can compute the multiplicity
+
 				Integer mult = intersection_multiplicity_via_flats(
 						rank_one_vectors, a_star.first, a_star.second, b_star.first, b_star.second);
 
@@ -438,7 +438,7 @@ namespace polymake { namespace tropical {
 				}
 			}//END iterate intersection rays
 
-			//If no points remain, return the empty_cycle 
+			//If no points remain, return the empty_cycle
 			if(nonfar_and_nonzero.size() == 0) return empty_cycle<Addition>(ambient_dim);
 			ab_vertices = ab_vertices.minor(nonfar_and_nonzero,~scalar2set(0));
 
@@ -450,12 +450,12 @@ namespace polymake { namespace tropical {
 
 
 	UserFunctionTemplate4perl("# @category Intersection theory"
-			"# Computes the intersection product of two cycles in a smooth surface"
-			"# @param Cycle<Addition> surface A smooth surface" 
-			"# @param Cycle<Addition> A any cycle in the surface"
-			"# @param Cycle<Addition> B any cycle in the surface"
-			"# @return Cycle<Addition> The intersection product of A and B in the surface",
-			"intersect_in_smooth_surface<Addition>(Cycle<Addition>,Cycle<Addition>, Cycle<Addition>)");
+                                  "# Computes the intersection product of two cycles in a smooth surface"
+                                  "# @param Cycle<Addition> surface A smooth surface"
+                                  "# @param Cycle<Addition> A any cycle in the surface"
+                                  "# @param Cycle<Addition> B any cycle in the surface"
+                                  "# @return Cycle<Addition> The intersection product of A and B in the surface",
+                                  "intersect_in_smooth_surface<Addition>(Cycle<Addition>,Cycle<Addition>, Cycle<Addition>)");
 
 	FunctionTemplate4perl("compute_surface_star<Addition>(Vector, Matrix,Matrix,SparseMatrix<Int>, IncidenceMatrix, Matrix, Matrix,IncidenceMatrix)");
 
