@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2015
+/* Copyright (c) 1997-2018
    Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
    http://www.polymake.org
 
@@ -33,15 +33,18 @@ class Matrix_base {
 protected:
    struct dim_t {
       int dimr, dimc;
+
       dim_t() : dimr(0), dimc(0) {}
-      dim_t(int r, int c) : dimr(c? r:0), dimc(r? c:0) {
+
+      dim_t(int r, int c) : dimr(r), dimc(c) {
          if (POLYMAKE_DEBUG) {
             if (dimr<0 || dimc<0)
                throw std::runtime_error("Matrix_base::dim_t out of range");
          }
       }
    };
-   typedef shared_array<E, list( PrefixData<dim_t>, AliasHandler<shared_alias_handler> ) > shared_array_type;
+
+   typedef shared_array<E, PrefixDataTag<dim_t>, AliasHandlerTag<shared_alias_handler>> shared_array_type;
    shared_array_type data;
 
    friend Matrix_base& make_mutable_alias(Matrix_base& alias, Matrix_base& owner)
@@ -49,43 +52,56 @@ protected:
       alias.data.make_mutable_alias(owner.data);
       return alias;
    }
-   template <bool,typename> friend class matrix_line_factory;
+   template <bool, typename> friend class matrix_line_factory;
 
    Matrix_base() {}
 
-   Matrix_base(int r, int c) : data(dim_t(r,c), r*c) {}
+   Matrix_base(int r, int c)
+      : data(dim_t(r,c), r*c) {}
 
-   template <typename Iterator>
-   Matrix_base(int r, int c, Iterator src) : data(dim_t(r,c), r*c, src) {}
+   template <typename... TArgs>
+   Matrix_base(int r, int c, TArgs&&... args)
+      : data(dim_t(r,c), r*c, std::forward<TArgs>(args)...) {}
 
-   Matrix_base(void *place, int r, int c) : data(place, dim_t(r,c), r*c) {}
+   Matrix_base(const shared_array_placement& place, int r, int c)
+      : data(place, dim_t(r,c), r*c) {}
 
-   template <typename Iterator>
-   Matrix_base(void *place, int r, int c, Iterator src) : data(place, dim_t(r,c), r*c, src) {}
+   template <typename... TArgs>
+   Matrix_base(const shared_array_placement& place, int r, int c, TArgs&&... args)
+      : data(place, dim_t(r,c), r*c, std::forward<TArgs>(args)...) {}
 
    friend class ConcatRows<Matrix_base>;
    template <typename, int> friend class alias;
 };
 
-   /**
-     @class Matrix
-     @brief \ref matrix_sec "Matrix type" class which holds the elements in a contiguous array <br>
-     Additional arithmetic operations for matrices and useful constructions (<code>unit_matrix, diag, ...</code>) are listed at @ref genericMatrices "operations".
-   */
+/**
+   @class Matrix
+   @brief \ref matrix_sec "Matrix type" class which holds the elements in a contiguous array <br>
+   Additional arithmetic operations for matrices and useful constructions (<code>unit_matrix, diag, ...</code>) are listed at @ref genericMatrices "operations".
+*/
 
 template <typename E>
-class Matrix :
-   public GenericMatrix<Matrix<E>, E>,
-   protected Matrix_base<E>
+class Matrix
+   : public GenericMatrix<Matrix<E>, E>
+   , protected Matrix_base<E>
 {
-   typedef Matrix_base<E> base;
-   typedef typename base::dim_t dim_t;
+   typedef Matrix_base<E> base_t;
+   using typename base_t::dim_t;
 
    friend Matrix& make_mutable_alias(Matrix& alias, Matrix& owner)
    {
-      return static_cast<Matrix&>(make_mutable_alias(static_cast<base&>(alias), static_cast<base&>(owner)));;
+      return static_cast<Matrix&>(make_mutable_alias(static_cast<base_t&>(alias), static_cast<base_t&>(owner)));;
    }
+
+protected:
+   template <typename Iterator>
+   struct fits_as_input_iterator
+      : bool_constant<(assess_iterator_value<Iterator, can_initialize, E>::value ||
+                       assess_iterator_value<Iterator, can_initialize, Vector<E>>::value)> {};
+
 public:
+   using typename GenericMatrix<Matrix>::generic_type;
+
    typedef E value_type;
    typedef E& reference;
    typedef const E& const_reference;
@@ -94,98 +110,63 @@ public:
    Matrix() {}
 
    /// create matrix with r rows and c columns, initialize all elements to 0
-   Matrix(int r, int c) : base(r,c) {}
+   Matrix(int r, int c)
+      : base_t(r,c) {}
 
-   template <typename E2>
-   Matrix(int r, int c, const E2& init,
-          typename enable_if<void**, convertible_to<E2, E>::value>::type=0)
-      : base(r, c, constant(init).begin()) {}
+   template <typename E2,
+             typename=typename std::enable_if<can_initialize<E2, E>::value>::type>
+   Matrix(std::initializer_list<std::initializer_list<E2>> l)
+      : base_t(l.size(), count_columns(l), l.begin()) {}
 
-   template <typename E2>
-   Matrix(int r, int c, const E2& init,
-          typename enable_if<void**, explicitly_convertible_to<E2, E>::value>::type=0)
-      : base(r, c, constant(conv<E2, E>(init)).begin()) {}
+   /// Create a matrix with given dimensions.  Elements are initialized from one or more input sequences.
+   /// Elements are assumed to come in the row order.
+   template <typename... Iterator, typename=typename std::enable_if<mlist_and_nonempty<fits_as_input_iterator<Iterator>...>::value>::type>
+   Matrix(int r, int c, Iterator&&... src)
+      : base_t(r, c, ensure_private_mutable(std::forward<Iterator>(src))...) {}
 
-   template <typename E2, size_t r, size_t c>
-   explicit Matrix(const E2 (&a)[r][c],
-                   typename enable_if<void**, convertible_to<E2, E>::value>::type=0)
-      : base(r, c, &a[0][0]) {}
-
-   template <typename E2, size_t r, size_t c>
-   explicit Matrix(const E2 (&a)[r][c],
-                   typename enable_if<void**, explicitly_convertible_to<E2, E>::value>::type=0)
-      : base(r, c, make_converting_iterator<E>(&a[0][0])) {}
-
-   /** Create a matrix with given dimensions.  Elements are initialized from an input sequence.
-       Elements are assumed to come in the row order.
-   */
-   template <typename Iterator>
-   Matrix(int r, int c, Iterator src,
-#ifdef __clang__
-          // clang tries to instantiate this contructor for SharedMatrix, where the first argument is void* .
-          typename disable_if<void**, std::numeric_limits<Iterator>::is_specialized>::type=0,
-#endif
-          typename construct_cascaded_iterator<Iterator, E, dense>::enabled=0)
-      : base(r, c, construct_cascaded_iterator<Iterator, E, dense>()(src)) {}
+   /// Create a matrix with given dimensions.  Elements are moved from one or more input sequences.
+   template <typename... Iterator, typename=typename std::enable_if<mlist_and_nonempty<fits_as_input_iterator<Iterator>...>::value>::type>
+   Matrix(int r, int c, polymake::operations::move, Iterator&&... src)
+      : base_t(r, c, polymake::operations::move(), std::forward<Iterator>(src)...) {}
 
    /// Copy of a disguised Matrix object.
-   Matrix(const GenericMatrix<Matrix>& m) : base(m.top()) {}
+   Matrix(const GenericMatrix<Matrix>& m)
+      : base_t(m.top()) {}
 
    /// Copy of an abstract matrix of the same element type.
    template <typename Matrix2>
    Matrix(const GenericMatrix<Matrix2, E>& m)
-      : base(m.rows(), m.cols(), ensure(concat_rows(m), (dense*)0).begin()) {}
+      : base_t(m.rows(), m.cols(), ensure(concat_rows(m), (dense*)0).begin()) {}
 
    /// Copy of an abstract matrix with element conversion.
    template <typename Matrix2, typename E2>
    explicit Matrix(const GenericMatrix<Matrix2, E2>& m,
-                   typename enable_if<void**, convertible_to<E2, E>::value>::type=0)
-      : base(m.rows(), m.cols(), ensure(concat_rows(m), (dense*)0).begin()) {}
-
-   template <typename Matrix2, typename E2>
-   explicit Matrix(const GenericMatrix<Matrix2, E2>& m,
-                   typename enable_if<void**, explicitly_convertible_to<E2, E>::value>::type=0)
-      : base(m.rows(), m.cols(), ensure(attach_converter<E>(concat_rows(m)), (dense*)0).begin()) {}
+                   typename std::enable_if<can_initialize<E2, E>::value>::type** = nullptr)
+      : base_t(m.rows(), m.cols(), ensure(concat_rows(m), (dense*)0).begin()) {}
 
    template <typename Container>
    explicit Matrix(const Container& src,
-                   typename construct_cascaded_iterator<typename Container::const_iterator, E, dense>::enabled=0,
-                   typename enable_if<void**, construct_cascaded_iterator<typename Container::const_iterator, E, dense>::depth==2>::type=0)
-      : base(src.size(), src.empty() ? 0 : get_dim(src.front()),
-             construct_cascaded_iterator<typename Container::const_iterator, E, dense>()(entire(src))) {}
+                   typename std::enable_if<isomorphic_to_container_of<Container, Vector<E>>::value>::type** = nullptr)
+      : base_t(src.size(), src.empty() ? 0 : get_dim(src.front()), src.begin()) {}
 
 protected:
-   Matrix(void *place, int r, int c) : base(place,r,c) {}
+   Matrix(const shared_array_placement& place, int r, int c)
+      : base_t(place,r,c) {}
 
    template <typename Iterator>
-   Matrix(void *place, int r, int c, Iterator src,
-          typename construct_cascaded_iterator<Iterator, E, dense>::enabled=0)
-      : base(place, r, c, construct_cascaded_iterator<Iterator, E, dense>()(src)) {}
+   Matrix(const shared_array_placement& place, int r, int c, Iterator&& src)
+      : base_t(place, r, c, std::forward<Iterator>(src)) {}
 
-   template <typename Container>
-   Matrix(void *place, int r, int c, const Container& src,
-          typename construct_cascaded_iterator<typename Container::const_iterator, E, dense>::enabled=0,
-          typename enable_if<void**, construct_cascaded_iterator<typename Container::const_iterator, E, dense>::depth==2>::type=0)
-      : base(place, r, c, construct_cascaded_iterator<typename Container::const_iterator, E, dense>()(entire(src))) {}
-
-   void resize(void *place, int r, int c)
+   void resize(const shared_array_placement& place, int r, int c)
    {
-      this->data.resize(place,r*c);
+      this->data.resize(place, r*c);
       this->data.get_prefix().dimr=r;
       this->data.get_prefix().dimc=c;
    }
 public:
 
-   /// Persistent matrix objects have after the assignment the same dimensions as the right hand side operand. 
-   /// Alias objects, such as matrix minor or block matrix, cannot be resized, thus must have the same dimensions as on the right hand side.
    Matrix& operator= (const Matrix& other) { assign(other); return *this; }
-#ifdef __clang__
-   template <typename Matrix2>
-   typename Matrix::generic_type::template enable_if_assignable_from<Matrix2>::type&
-   operator= (const GenericMatrix<Matrix2>& other) { return Matrix::generic_type::operator=(other); }
-#else
-   using Matrix::generic_type::operator=;
-#endif
+   using generic_type::operator=;
 
    /// Exchange the contents of two matrices in a most efficient way. 
    /// If at least one non-persistent object is involved, the operands must have equal dimensions. 
@@ -216,19 +197,20 @@ public:
    }
 
    template <typename E2>
-   typename enable_if<void, convertible_to<E2, E>::value>::type
+   typename std::enable_if<can_initialize<E2, E>::value, void>::type
    assign(int r, int c, const E2& x)
    {
-      this->data.assign(r*c, constant(x).begin());
+      this->data.assign(r*c, x);
       this->data.get_prefix()=dim_t(r,c);
    }
 
    /// Truncate to 0x0 matrix.
    void clear() { this->data.clear(); }
+
    void clear(int r, int c)
    {
       this->data.resize(r*c);
-      this->data.get_prefix()=dim_t(r,c);
+      this->data.enforce_unshared().get_prefix()=dim_t(r,c);
    }
 
    /// the number of rows of the matrix
@@ -261,7 +243,7 @@ protected:
    void assign(const GenericMatrix<Matrix2>& m)
    {
       const int r=m.rows(), c=m.cols();
-      this->data.assign(r*c, ensure(concat_rows(convert_to<E>(m)), (dense*)0).begin());
+      this->data.assign(r*c, ensure(concat_rows(m), (dense*)0).begin());
       this->data.get_prefix().dimr=r;
       this->data.get_prefix().dimc=c;
    }
@@ -278,39 +260,38 @@ protected:
       this->data.assign_op(ensure(concat_rows(src2), (dense*)0).begin(), op);
    }
 
+   // TODO: provide moving version of the following 4 methods
    template <typename Matrix2, typename E2>
    void append_rows(const GenericMatrix<Matrix2, E2>& m)
    {
-      this->data.append(concat_rows(m).dim(), ensure(concat_rows(convert_to<E>(m)), (dense*)0).begin());
+      this->data.append(concat_rows(m).dim(), ensure(concat_rows(m), (dense*)0).begin());
       this->data.get_prefix().dimr+=m.rows();
    }
 
    template <typename Vector2>
    void append_row(const GenericVector<Vector2>& v)
    {
-      this->data.append(v.dim(), ensure(convert_to<E>(v), (dense*)0).begin());
+      this->data.append(v.dim(), ensure(v.top(), (dense*)0).begin());
       this->data.get_prefix().dimr++;
    }
 
    template <typename Matrix2>
    void append_cols(const GenericMatrix<Matrix2>& m)
    {
-      this->data.weave(m.rows()*m.cols(), this->cols(), pm::rows(convert_to<E>(m)).begin());
+      this->data.weave(m.rows()*m.cols(), this->cols(), pm::rows(m).begin());
       this->data.get_prefix().dimc+=m.cols();
    }
 
    template <typename Vector2>
    void append_col(const GenericVector<Vector2>& v)
    {
-      this->data.weave(v.dim(), this->cols(), ensure(convert_to<E>(v), (dense*)0).begin());
-      this->data.get_prefix().dimc++;
+      append_cols(vector2col(v));
    }
 
    template <typename E2>
-   typename enable_if<void, convertible_to<E2, E>::value>::type
-   _fill(const E2& x, True)
+   void fill_impl(const E2& x, std::true_type)
    {
-      this->data.assign(this->data.size(), constant(x).begin());
+      this->data.assign(this->data.size(), x);
    }
 
    void stretch_rows(int r)
@@ -332,7 +313,7 @@ protected:
 };
 
 template <typename E>
-struct check_container_feature<Matrix<E>, FlatStorage> : True {};
+struct check_container_feature<Matrix<E>, FlatStorage> : std::true_type {};
 
 template <typename E>
 class ConcatRows< Matrix_base<E> >
@@ -342,10 +323,13 @@ protected:
    ~ConcatRows();
    Matrix_base<E>& hidden() { return *reinterpret_cast<Matrix_base<E>*>(this); }
    const Matrix_base<E>& hidden() const { return *reinterpret_cast<const Matrix_base<E>*>(this); }
+
+   friend class plain_array< ConcatRows< Matrix_base<E> >, E >;
+
+   E* get_data() { return *hidden().data; }
+   const E* get_data() const { return *hidden().data; }
 public:
    int size() const { return hidden().data.size(); }
-   E& front() { return **hidden().data; }
-   const E& front() const { return **hidden().data; }
 
    ConcatRows& operator= (const ConcatRows& other) { return ConcatRows::generic_type::operator=(other); }
    using ConcatRows::generic_type::operator=;
@@ -388,10 +372,10 @@ struct binary_op_builder< matrix_line_factory<rowwise>, Iterator1, Iterator2, Re
 template <typename E>
 class Rows< Matrix<E> >
    : public modified_container_pair_impl< Rows< Matrix<E> >,
-                                          list( Container1< constant_value_container< Matrix_base<E>& > >,
-                                                Container2< series >,
-                                                Operation< matrix_line_factory<true> >,
-                                                MasqueradedTop ) > {
+                                          mlist< Container1Tag< constant_value_container< Matrix_base<E>& > >,
+                                                 Container2Tag< series >,
+                                                 OperationTag< matrix_line_factory<true> >,
+                                                 MasqueradedTop > > {
 protected:
    ~Rows();
 public:
@@ -418,10 +402,10 @@ public:
 template <typename E>
 class Cols< Matrix<E> >
    : public modified_container_pair_impl< Cols< Matrix<E> >,
-                                          list( Container1< constant_value_container< Matrix_base<E>& > >,
-                                                Container2< sequence >,
-                                                Operation< matrix_line_factory<false> >,
-                                                MasqueradedTop ) > {
+                                          mlist< Container1Tag< constant_value_container< Matrix_base<E>& > >,
+                                                 Container2Tag< sequence >,
+                                                 OperationTag< matrix_line_factory<false> >,
+                                                 MasqueradedTop > > {
 protected:
    ~Cols();
 public:
@@ -444,53 +428,53 @@ public:
    }
 };
 
-template <typename MatrixTop, typename E, typename Permutation> inline
-typename enable_if<Matrix<E>, (MatrixTop::is_nonsymmetric && !MatrixTop::is_sparse)>::type
-permuted_rows(const GenericMatrix<MatrixTop,E>& m, const Permutation& perm)
+template <typename TMatrix, typename E, typename Permutation> inline
+typename std::enable_if<TMatrix::is_nonsymmetric && !TMatrix::is_sparse, Matrix<E>>::type
+permuted_rows(const GenericMatrix<TMatrix, E>& m, const Permutation& perm)
 {
-   if (POLYMAKE_DEBUG || !Unwary<MatrixTop>::value) {
+   if (POLYMAKE_DEBUG || !Unwary<TMatrix>::value) {
       if (m.rows() != perm.size())
          throw std::runtime_error("permuted_rows - dimension mismatch");
    }
    return Matrix<E>(m.rows(), m.cols(), select(rows(m),perm).begin());
 }
 
-template <typename MatrixTop, typename E, typename Permutation> inline
-typename enable_if<Matrix<E>, (MatrixTop::is_nonsymmetric && !MatrixTop::is_sparse)>::type
-permuted_cols(const GenericMatrix<MatrixTop,E>& m, const Permutation& perm)
+template <typename TMatrix, typename E, typename Permutation> inline
+typename std::enable_if<TMatrix::is_nonsymmetric && !TMatrix::is_sparse, Matrix<E>>::type
+permuted_cols(const GenericMatrix<TMatrix, E>& m, const Permutation& perm)
 {
-   if (POLYMAKE_DEBUG || !Unwary<MatrixTop>::value) {
+   if (POLYMAKE_DEBUG || !Unwary<TMatrix>::value) {
       if (m.cols() != perm.size())
          throw std::runtime_error("permuted_cols - dimension mismatch");
    }
    Matrix<E> result(m.rows(), m.cols());
-   copy(entire(select(cols(m),perm)), cols(result).begin());
+   copy_range(entire(select(cols(m),perm)), cols(result).begin());
    return result;
 }
 
-template <typename MatrixTop, typename E, typename Permutation> inline
-typename enable_if<Matrix<E>, (MatrixTop::is_nonsymmetric && !MatrixTop::is_sparse)>::type
-permuted_inv_rows(const GenericMatrix<MatrixTop,E>& m, const Permutation& perm)
+template <typename TMatrix, typename E, typename Permutation> inline
+typename std::enable_if<TMatrix::is_nonsymmetric && !TMatrix::is_sparse, Matrix<E>>::type
+permuted_inv_rows(const GenericMatrix<TMatrix, E>& m, const Permutation& perm)
 {
-   if (POLYMAKE_DEBUG || !Unwary<MatrixTop>::value) {
+   if (POLYMAKE_DEBUG || !Unwary<TMatrix>::value) {
       if (m.rows() != perm.size())
          throw std::runtime_error("permuted_inv_rows - dimension mismatch");
    }
    Matrix<E> result(m.rows(), m.cols());
-   copy(entire(rows(m)), select(rows(result), perm).begin());
+   copy_range(entire(rows(m)), select(rows(result), perm).begin());
    return result;
 }
 
-template <typename MatrixTop, typename E, typename Permutation> inline
-typename enable_if<Matrix<E>, (MatrixTop::is_nonsymmetric && !MatrixTop::is_sparse)>::type
-permuted_inv_cols(const GenericMatrix<MatrixTop,E>& m, const Permutation& perm)
+template <typename TMatrix, typename E, typename Permutation> inline
+typename std::enable_if<TMatrix::is_nonsymmetric && !TMatrix::is_sparse, Matrix<E>>::type
+permuted_inv_cols(const GenericMatrix<TMatrix, E>& m, const Permutation& perm)
 {
-   if (POLYMAKE_DEBUG || !Unwary<MatrixTop>::value) {
+   if (POLYMAKE_DEBUG || !Unwary<TMatrix>::value) {
       if (m.cols() != perm.size())
          throw std::runtime_error("permuted_inv_cols - dimension mismatch");
    }
    Matrix<E> result(m.rows(), m.cols());
-   copy(entire(cols(m)), select(cols(result), perm).begin());
+   copy_range(entire(cols(m)), select(cols(result), perm).begin());
    return result;
 }
 
