@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2016
+/* Copyright (c) 1997-2018
    Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
    http://www.polymake.org
 
@@ -79,13 +79,6 @@ END_EXTERN_C
 
 #include "polymake/perl/constants.h"
 
-extern I32 pm_perl_skip_debug_cx;
-#define SkipDebugSub(cv) (pm_perl_skip_debug_cx && CvSTASH(cv)==PL_debstash)
-#define SkipDebugFrame(cx,plus_level) (pm_perl_skip_debug_cx && ((plus_level && CvSTASH(cx->blk_sub.cv)==PL_debstash) || CopSTASH_eq(cx->blk_oldcop,PL_debstash)))
-
-// CvROOT and CvXSUB are in the same union
-#define IsWellDefinedSub(x)   (CvROOT(x) != NULL)
-
 // PerlVersion < 5180
 #ifndef PadlistARRAY
 # define PadlistARRAY(x) ((AV**)AvARRAY(x))
@@ -124,88 +117,177 @@ extern I32 pm_perl_skip_debug_cx;
 # define PmEmptyArraySlot &PL_sv_undef
 #endif
 
-// these values have to be checked in toke.c for each new perl release
-#if PerlVersion < 5250
-# define LEX_KNOWNEXT 0
-#endif
-#define LEX_NORMAL 10
-
 // check whether this private flag is not used in OP_METHOD_NAMED for each new perl release
 #define MethodIsCalledOnLeftSideOfArrayAssignment 1
 
-START_EXTERN_C
+namespace pm { namespace perl { namespace glue {
 
-CV* pm_perl_get_cur_cv(pTHX);
-SV **pm_perl_get_cx_curpad(pTHX_ PERL_CONTEXT* cx, PERL_CONTEXT* cx_bottom);
+// public export from Ext
+extern bool skip_debug_cx;
+
+inline bool skip_debug_sub(pTHX_ CV* cv)
+{
+   return skip_debug_cx && CvSTASH(cv)==PL_debstash;
+}
+
+inline bool skip_debug_frame(pTHX_ PERL_CONTEXT* cx)
+{
+   return skip_debug_cx && CopSTASH_eq(cx->blk_oldcop, PL_debstash);
+}
+
+CV* get_cur_cv(pTHX);
+SV** get_cx_curpad(pTHX_ PERL_CONTEXT* cx, PERL_CONTEXT* cx_bottom);
 
 // public export from Poly
-OP* pm_perl_select_method_helper_op(pTHX);
-MAGIC* pm_perl_array_flags_magic(pTHX_ SV*);
-SV* pm_perl_name_of_ret_var(pTHX);
-int pm_perl_canned_dup(pTHX_ MAGIC* mg, CLONE_PARAMS* param);
+OP* select_method_helper_op(pTHX);
+MAGIC* array_flags_magic(pTHX_ SV*);
+SV* name_of_ret_var(pTHX);
+int canned_dup(pTHX_ MAGIC* mg, CLONE_PARAMS* param);
 
 // public export from RefHash
-HE* pm_perl_refhash_fetch_ent(pTHX_ HV* hv, SV* keysv, I32 lval);
+HE* refhash_fetch_ent(pTHX_ HV* hv, SV* keysv, I32 lval);
 
 // public export from namespaces
-SV* pm_perl_namespace_try_lookup(pTHX_ HV* stash, SV* name, I32 type);
-HV* pm_perl_namespace_lookup_class(pTHX_ HV* stash, const char* class_name, STRLEN class_namelen, int lex_lookup_ix);
-HV* pm_perl_namespace_lookup_class_autoload(pTHX_ HV* stash, const char* class_name, STRLEN class_namelen, int lex_lookup_ix);
-CV* pm_perl_namespace_lookup_sub(pTHX_ HV* stash, const char* name, STRLEN namelen, CV* lex_context_cv);
-typedef void (*namespace_plugin_fun_ptr)(pTHX_ SV*);
-void pm_perl_namespace_register_plugin(pTHX_ namespace_plugin_fun_ptr enabler, namespace_plugin_fun_ptr disabler, SV *data);
+SV* namespace_try_lookup(pTHX_ HV* stash, SV* name, I32 type);
+HV* namespace_lookup_class(pTHX_ HV* stash, const char* class_name, STRLEN class_namelen, int lex_lookup_ix, bool override_negative_cache=false);
+HV* namespace_lookup_class_autoload(pTHX_ HV* stash, const char* class_name, STRLEN class_namelen, int lex_lookup_ix);
+CV* namespace_lookup_sub(pTHX_ HV* stash, const char* name, STRLEN namelen, CV* lex_context_cv);
+using namespace_plugin_fun_ptr=void (*)(pTHX_ SV*);
+void namespace_register_plugin(pTHX_ namespace_plugin_fun_ptr enabler, namespace_plugin_fun_ptr disabler, SV* data);
 
 // public export from Scope
-void pm_perl_localize_scalar(pTHX_ SV* var);
-void pm_perl_localize_array(pTHX_ SV* av, SV* ar_ref);
+void localize_scalar(pTHX_ SV* var);
+void localize_array(pTHX_ SV* av, SV* ar_ref);
  
 // public export from CPlusPlus
-OP* pm_perl_cpp_helem(pTHX_ HV* hv, const MAGIC* mg);
-OP* pm_perl_cpp_hslice(pTHX_ HV* hv, const MAGIC* mg);
-OP* pm_perl_cpp_exists(pTHX_ HV* hv, const MAGIC* mg);
-OP* pm_perl_cpp_delete_hslice(pTHX_ HV* hv, const MAGIC* mg);
-OP* pm_perl_cpp_delete_helem(pTHX_ HV* hv, const MAGIC* mg);
-OP* pm_perl_cpp_keycnt(pTHX_ HV* hv, const MAGIC* mg);
-int pm_perl_cpp_hassign(pTHX_ HV* hv, MAGIC* mg, I32* firstRp, I32 lastR, int return_size);
-int pm_perl_cpp_has_assoc_methods(const MAGIC* mg);
+OP* cpp_helem(pTHX_ HV* hv, const MAGIC* mg);
+OP* cpp_hslice(pTHX_ HV* hv, const MAGIC* mg);
+OP* cpp_exists(pTHX_ HV* hv, const MAGIC* mg);
+OP* cpp_delete_hslice(pTHX_ HV* hv, const MAGIC* mg);
+OP* cpp_delete_helem(pTHX_ HV* hv, const MAGIC* mg);
+OP* cpp_keycnt(pTHX_ HV* hv, const MAGIC* mg);
+int cpp_hassign(pTHX_ HV* hv, MAGIC* mg, I32* firstRp, I32 lastR, int return_size);
+int cpp_has_assoc_methods(const MAGIC* mg);
 
-END_EXTERN_C
+// public export from Struct
+SV* retrieve_pkg(pTHX_ SV* obj);
+HV* retrieve_pkg_stash(pTHX_ SV* obj);
 
-#ifndef __cplusplus
-static inline
-I32 min (I32 a, I32 b) { return a<b ? a : b; }
+// CvROOT and CvXSUB are in the same union
+inline bool is_well_defined_sub(CV* x)
+{
+   return CvROOT(x) != Nullop;
+}
 
-static inline
-I32 max (I32 a, I32 b) { return a>b ? a : b; }
-
-static inline
+inline
 void write_protect_on(pTHX_ SV* x)
 {
    if (x != &PL_sv_undef) SvREADONLY_on(x);
 }
 
-static inline
+inline
 void write_protect_off(pTHX_ SV* x)
 {
    if (x != &PL_sv_undef) SvREADONLY_off(x);
 }
 
 // for given OP_ENTERSUB, find the corresponding OP_METHOD_NAMED, or return NULL
-static inline
+inline
 OP* method_named_op(OP* o)
 {
    return ((o->op_flags & OPf_KIDS) && (o=cLISTOPo->op_last) && o->op_type == OP_METHOD_NAMED) ? o : 0;
 }
-#endif
 
-static inline
-MAGIC* pm_perl_get_cpp_magic(SV* sv)
+inline
+MAGIC* get_cpp_magic(SV* sv)
 {
    MAGIC* mg;
-   for (mg=SvMAGIC(sv); mg && mg->mg_virtual->svt_dup != &pm_perl_canned_dup; mg=mg->mg_moremagic) ;
+   for (mg=SvMAGIC(sv); mg && mg->mg_virtual->svt_dup != &canned_dup; mg=mg->mg_moremagic) ;
    return mg;
 }
 
+inline
+HV* get_cached_stash(pTHX_ SV* pkg)
+{
+   if (!pkg) return nullptr;
+   if (!(SvFLAGS(pkg) & SVf_IVisUV)) {
+      HV* stash=gv_stashsv(pkg, GV_NOADD_NOINIT);
+#if PerlVersion < 5180
+      // perl 5.16 destroys these flags in SvUPGRADE
+      // which has catastrophic consequences for use of such constants in reentrant subs
+      // (they are not propagated into PAD clones)
+      const auto save_flags = SvFLAGS(pkg) & (SVf_FAKE | SVf_READONLY);
+      SvFLAGS(pkg) &= ~(SVf_FAKE | SVf_READONLY);
+#endif
+      SvUPGRADE(pkg, SVt_PVIV);
+      SvUV_set(pkg, reinterpret_cast<UV>(stash));
+#if PerlVersion < 5180
+      SvFLAGS(pkg) |= SVf_IVisUV | save_flags;
+#else
+      SvFLAGS(pkg) |= SVf_IVisUV;
+#endif
+   }
+   return reinterpret_cast<HV*>(SvUVX(pkg));
+}
+
+// for sprintf and similar using a "%.*s" format
+#define PmPrintGvNAME(gv) (int)GvNAMELEN(gv), GvNAME(gv)
+#define PmPrintHvNAME(hv) (int)HvNAMELEN(hv), HvNAME(hv)
+
+// a sort of light-weight unique_ptr for OPs under construction
+template <typename OpType>
+class op_keeper {
+public:
+   op_keeper(pTHX_ OpType* o)
+      : PmPerlInterpreterMemberInit(pi)
+        op(o) {}
+
+   op_keeper& operator= (OpType* o)
+   {
+      op=o;
+      return *this;
+   }
+
+   operator OpType* () const { return op; }
+   OpType* operator-> () const { return op; }
+   explicit operator bool () const { return op != nullptr; }
+
+   OpType* release()
+   {
+      OpType* o=op;
+      op=nullptr;
+      return o;
+   }
+
+   ~op_keeper()
+   {
+      dTHXa(pi);
+      if (op) op_free(op);
+   }
+
+   op_keeper(op_keeper&&)=default;
+private:
+   op_keeper(const op_keeper&)=delete;
+   op_keeper& operator=(const op_keeper&)=delete;
+
+   PmPerlInterpreterMemberDecl(pi);
+   OpType* op;
+};
+
+#if defined(NDEBUG) || PerlVersion >= 5220
+# define PmNewCustomOP(type, ...) new##type(OP_CUSTOM, __VA_ARGS__)
+#else
+// old perls have a too restrictive assertion denying creation of custom operations of types other than BASEOP.
+inline OP* set_op_type(OP* o, uint32_t type)
+{
+   o->op_type=type;
+   return o;
+}
+# define PmNewCustomOP(type, ...) set_op_type(new##type((OA_##type==OA_SVOP ? OP_CONST : OP_NULL), __VA_ARGS__), OP_CUSTOM)
+#endif
+
+} } }
+
 #endif // POLYMAKE_PERL_EXT_H
 
 // Local Variables:
