@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2016
+/* Copyright (c) 1997-2018
    Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
    http://www.polymake.org
 
@@ -273,6 +273,13 @@ bool Integer::fill_from_file(int fd)
    return true;
 }
 
+template <>
+Integer
+pow(const Integer& base, int exp)
+{
+   return Integer::pow(base,exp);
+}
+
 namespace {
 
 mp_limb_t limb0=0, limb1=1;
