@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2015
+/* Copyright (c) 1997-2018
    Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
    http://www.polymake.org
 
@@ -170,7 +170,7 @@ int socketbuf::sync()
       out_size-=written;
       start+=written;
    }
-   setp(pbase(),epptr());
+   setp(pbase(), epptr());
    return 0;
 }
 
@@ -302,7 +302,8 @@ server_socketbuf::server_socketbuf(const char* path)
    if ((_wfd=_fd=socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
       throw std::runtime_error(std::string("server_socketbuf: socket failed: ") += strerror(errno));
    sockaddr_un sa = { AF_UNIX };
-   strncpy(sa.sun_path, path, sizeof(sa.sun_path));
+   strncpy(sa.sun_path, path, sizeof(sa.sun_path)-1);
+   sa.sun_path[sizeof(sa.sun_path)-1] = '\0';
    if (bind(_fd, (sockaddr*)&sa, sizeof(sa)))
       throw std::runtime_error(std::string("server_socketbuf: bind failed: ") += strerror(errno));
    fcntl(_fd, F_SETFD, FD_CLOEXEC);
@@ -320,6 +321,11 @@ int socketstream::port() const
    return ntohs(sa.sin_port);
 }
 
+void socketbuf::discard_out()
+{
+   setp(pbase(), epptr());
+}
+
 inline
 socketbuf* server_socketbuf::start(server_socketbuf *me)
 {
