@@ -1,4 +1,4 @@
-#  Copyright (c) 1997-2015
+#  Copyright (c) 1997-2018
 #  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
 #  http://www.polymake.org
 #
@@ -15,6 +15,7 @@
 
 use strict;
 use namespaces;
+use warnings qw(FATAL void syntax misc);
 
 use Polymake::Enum qw( clock_start=100000000 );
 my $clock=$clock_start;
@@ -125,14 +126,7 @@ sub DESTROY {
    }
 }
 
-END {
-   if ($] >= 5.012) {
-      undef &DESTROY;
-   } else {
-      # perl 5.10.1 crashes when a destructor becomes undefined
-      *DESTROY=sub {}
-   }
-}
+END { undef &DESTROY; }
 ####################################################################################
 # return the items from a control list, removing duplicates
 sub get_items {
@@ -205,7 +199,7 @@ package Polymake::Core::Preference::Label;
 use Polymake::Struct (
    [ new => '$;$$$$' ],
    [ '$name' => '#1' ],
-   [ '$parent' => 'weak( #2 )' ],       # Label higher in the hierarchy
+   [ '$parent' => 'weak(#2)' ],         # Label higher in the hierarchy
    [ '$wildcard_name' => '#3 || "*"' ],
    '%children',                         # Labels lower in the hierarchy
    '%controls',                         # control list -> number of items in this list
@@ -596,7 +590,7 @@ sub belongs_to {
             $app->imported->{$label->application->name} or return 0;
          }
       } else {
-         warn_print( "label ", $label->full_name, " seems to be obsolete" );
+         warn_print( "label ", $label->full_name, " might to be obsolete" ) if $DeveloperMode && ! -d "$InstallTop/bundled/".($label->parent_name);
          return 0;
       }
    }
@@ -616,8 +610,8 @@ package Polymake::Core::Preference::perApplication;
 
 use Polymake::Struct (
    [ new => '$$' ],
-   [ '$handler' => 'weak( #1 )' ],
-   [ '$application' => 'weak( #2 )' ],
+   [ '$handler' => 'weak(#1)' ],
+   [ '$application' => 'weak(#2)' ],
    '@imported',                      # perApplication objects of imported applications
    '%labels',
    '@default_prefs',
@@ -628,7 +622,7 @@ use Polymake::Struct (
 ####################################################################################
 sub find_label {
    my ($self, $name, $create)=@_;
-   my ($name, @sublevels)=split /\./, $name;
+   ($name, my @sublevels)=split /\./, $name;
    my $label=$self->labels->{$name};
    unless ($label) {
       foreach (@{$self->imported}) {
@@ -715,6 +709,11 @@ sub add_preference {
 
    if ($mode & 2) {
       push @{$self->default_prefs}, $pref;
+      # activate this preference right now if the application
+      # is already active, otherwise end_loading will do this
+      if (contains($self->handler->applications,$self)) {
+         $self->handler->activate(0,$pref);
+      }
    } else {
       if (defined (my $dominating=$self->handler->check_repeating($pref))) {
          if ($mode==1) {
