@@ -1,4 +1,4 @@
-#  Copyright (c) 1997-2015
+#  Copyright (c) 1997-2018
 #  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
 #  http://www.polymake.org
 #
@@ -15,6 +15,7 @@
 
 use strict;
 use namespaces;
+use warnings qw(FATAL void syntax misc);
 
 package Polymake::Core::Rule;
 use POSIX qw( :signal_h :sys_wait_h );
@@ -61,6 +62,7 @@ use Polymake::Struct (
    [ '$dyn_weight' => 'undef' ],        # Rule  also inserted in preconditions
    [ '$credit' => '#4' ],               # optional Credit from the rule file
    [ '$with_permutation' => 'undef' ],  # CreatingPermutation
+   [ '$not_for_twin' => 'undef' ],      # Property->key of a twin property this rule has been flipped for
 );
 
 #  The ARRAY elements in the input lists may have following array flags (see get_array_flags),
@@ -84,21 +86,29 @@ sub without_permutation { $_[0] }
 # private:
 sub check_repeated_properties {
    my ($path, $seen, $tag)=@_;
-   my $last_limb;
+   my ($prop, $state);
    {
-      $last_limb=local_pop($path);
-      $seen=($seen->{$_->key} ||= { }) for @$path;
+      $prop=local_pop($path);
+      $seen=($seen->{$_->key} //= { }) for @$path;
    }
-   foreach my $prop ($last_limb->all_choices) {
-      if (my $check=$tag>0 ? $seen->{$prop->key}++ : $seen->{$prop->key}--) {
-         my $name=Property::print_path($path);
-         if ($tag>0) {
-            croak( "rule source '$name' occurs twice" );
-         } elsif ($check<0) {
-            croak( "rule target '$name' occurs twice" );
-         } else {
-            croak( "A rule may not have the property '$name' both as a source and a target" );
-         }
+   if ($tag != ($state=($seen->{$prop->key} += $tag))) {
+      my $name=Property::print_path($path);
+      if ($state>0) {
+         croak( "Rule source '$name' occurs twice" );
+      } elsif ($state<0) {
+         croak( "Rule target '$name' occurs twice" );
+      } else {
+         croak( "Property '$name' occurs both as a source and a target" );
+      }
+   }
+   if ($tag>0 and $prop->flags & $Property::is_multiple) {
+      croak( "A rule may not have an entire multiple subobject as a source but only its single properties" );
+   }
+   if ($prop->flags & $Property::is_permutation) {
+      if ($tag>0) {
+         croak( "A rule may not have an entire permuted subobject as a source but only its single properties" );
+      } else {
+         croak( "A permuted subobject can't be produced as a whole" );
       }
    }
 }
@@ -109,10 +119,7 @@ sub parse_input {
    @{$self->input}=map {
       my $input=$self->defined_for->encode_read_request($_);
       if ($Application::plausibility_checks) {
-         check_repeated_properties($input, $seen, 1);
-         if (grep { $_->flags & $Property::is_multiple } (is_object($input) ? $input : $input->[-1])->all_choices) {
-            croak( "A rule may not consume a multiple subobject as a whole but only its single properties" );
-         }
+         check_repeated_properties($_, $seen, 1) for @$input;
       }
       $input
    } split /\s*,\s*/, $sources;
@@ -133,20 +140,14 @@ sub parse_output {
 
       foreach (split /\./, $target) {
          if (my ($prop_name, $attrib)= /^($id_re) (?: \(\s* ($id_re) \s*\) )? $/xo) {
-            if (defined $prop) {
-               $proto=$prop->specialization($proto, 1)->type;
-            }
-            unless ($prop=$proto->lookup_property($prop_name)) {
-               $proto==$_[0]
-               ? croak( "Unknown property ", $_[0]->full_name, "::$prop_name" )
-               : croak( "Unknown property ", $proto->full_name, "::$_ in path $target" );
-            }
+            $prop=$proto->property($prop_name);
+            $proto=$prop->type;
             push @out, $prop;
             if ($prop->flags & $Property::is_multiple) {
                if (defined $attrib) {
                   if ($Application::plausibility_checks) {
                      foreach my $output (@{$self->output}) {
-                        my $l=equal_list_prefixes(\@out, $output);
+                        my $l=Property::equal_path_prefixes(\@out, $output);
                         if ($l==@out) {
                            $attrib eq "any"
                              or croak( "Contradicting attributes `any' and `new' for multiple subobject $prop_name in rule targets ",
@@ -335,15 +336,15 @@ sub append_precondition {
 }
 
 sub append_existence_check {
-   my ($self, $not, $req, $proto)=@_;
-   my $prop_list=$proto->encode_read_request($req);
+   my ($self, $not, $req_string, $proto)=@_;
+   my $req=$proto->encode_read_request($req_string);
    my $neg=length($not);
    my $precond=special Rule(':',
                             $neg
-                            ? sub { is_boolean_false($_[0]->lookup_request($prop_list, 1)) }  # forbid shortcuts
-                            : sub { is_object($_[0]->lookup_request($prop_list, 1)) },
+                            ? sub { !defined($_[0]) || is_boolean_false($_[0]->lookup_request($req, 1)) }  # forbid shortcuts
+                            : sub { defined($_[0]) && is_object($_[0]->lookup_request($req, 1)) },
                             $proto);
-   $precond->header="precondition : ${not}exists($req) ( " . $self->header . " )";
+   $precond->header="precondition : ${not}exists($req_string) ( " . $self->header . " )";
    $precond->flags=$is_precondition;
    push @{$self->preconditions}, $precond;
 }
@@ -365,51 +366,111 @@ sub append_weight {
 }
 
 sub append_permutation {
-   my ($self, $perm)=@_;
+   my ($self, $perm_name)=@_;
+   my @perm_path=$self->defined_for->encode_descending_path($perm_name);
    if ($Application::plausibility_checks) {
+      if (Property::find_first_in_path(\@perm_path, $Property::is_permutation) != $#perm_path) {
+         croak( "$perm_name is not a permutation" );
+      }
       if (defined $self->with_permutation) {
          croak( "Sorry, multiple permutations per production rule are not implemented" );
       }
    }
-   $self->with_permutation=new CreatingPermutation($self, $perm);
+   $self->with_permutation=new CreatingPermutation($self, bless \@perm_path, "Polymake::Core::Rule::PermutationPath");
 }
 ####################################################################################
-sub matching_input {
+sub has_matching_input {
    my ($self, $path)=@_;
    my $lp=@$path;
    foreach my $input (@{$self->input}) {
-      if ($lp==@$input) {
-         my $match=equal_list_prefixes($path, $input);
-         return $input if $match==$lp or $match+1==$lp && $input->[-1]->matches($path->[-1]);
+      foreach my $input_path (@$input) {
+        if ($lp==@$input_path) {
+           my $match=Property::equal_path_prefixes($path, $input_path);
+           return 1 if $match==$lp or $match+1==$lp && $input_path->[-1]->key == $path->[-1]->key;
+        }
       }
    }
-   undef
+   0
+}
+####################################################################################
+sub flip_path_for_twin {
+   my ($path, $proto, $twin_prop)=@_;
+   my $result;
+   if ($path->[0]->key == $twin_prop->key) {
+      $result=[ @$path[1..$#$path] ];
+   } elsif ($twin_prop->type->isa($path->[0]->defined_for)) {
+      $result=[ $twin_prop, @$path ];
+   } else {
+      return;
+   }
+   if (is_object($path)) {
+      inherit_class($result, $path);
+   } elsif (my $flags=get_array_flags($path)) {
+      set_array_flags($result, $flags);
+   }
+   $result
+}
+
+sub flip_rule_for_twin {
+   my ($self, $twin_prop)=@_;
+   my $flipped_input=[ map { [ map { flip_path_for_twin($_, $self->defined_for, $twin_prop) // return } @$_ ] } @{$self->input} ];
+   my $flipped_output= defined($self->output) ? [ map { flip_path_for_twin($_, $self->defined_for, $twin_prop) // return } @{$self->output} ] : undef;
+   my $flipped_perm_path= defined($self->with_permutation) ? flip_path_for_twin($self->with_permutation->perm_path, $self->defined_for, $twin_prop) // return : undef;
+   my $code=$self->code;
+   my $flipped_rule=_new($self, $self->header." (applied to ".$twin_prop->name.")",
+                         $code && ($self->flags & $is_precondition
+                                   ? sub { my ($this)=@_; $code->($this->descend([ $twin_prop, undef ])) }
+                                   : sub { my ($this)=@_; $code->($this->descend_and_create([ $twin_prop, undef ])) } ),
+                         $self->defined_for, $self->credit);
+   $flipped_rule->input=$flipped_input;
+   $flipped_rule->output=$flipped_output;
+   $flipped_rule->flags=$self->flags;
+   $flipped_rule->weight=$self->weight;
+   @{$flipped_rule->preconditions}=map { flip_rule_for_twin($_, $twin_prop) } @{$self->preconditions};
+   if (defined($self->dyn_weight)) {
+      $flipped_rule->dyn_weight=$flipped_rule->preconditions->[list_index($self->preconditions, $self->dyn_weight)];
+   }
+   if (defined($flipped_perm_path)) {
+      $flipped_rule->with_permutation=new CreatingPermutation($flipped_rule, $flipped_perm_path);
+   }
+   $flipped_rule->not_for_twin=$twin_prop->key;
+   $flipped_rule;
 }
 ####################################################################################
 sub needs_finalization {
    my ($self)=@_;
 
-   my $input_perm;
+   my @path_to_perm;
    foreach my $input (@{$self->input}) {
-      if (!is_object($input) and get_array_flags($input) & $Property::is_permutation) {
+      if (get_array_flags($input) & $Property::is_permutation) {
          if ($Application::plausibility_checks) {
-            my @perms_in_path=grep { $_->flags & $Property::is_permutation } @$input;
             if ($self->flags & $is_initial) {
                croak( "initial rules may not trigger permutations" );
             }
-            if (@perms_in_path>1 || defined($input_perm) && $input_perm != $perms_in_path[0]) {
-               croak( "Properties from several permutation subobjects occur as inputs in the same rule" );
+            foreach my $path (@$input) {
+               my $perm_index=Property::find_first_in_path($path, $Property::is_permutation);
+               if ($perm_index<0) {
+                  croak( "A permuted subobject mixed with non-permuted properties in one source group" );
+               }
+               if (@path_to_perm) {
+                  if (Property::equal_path_prefixes(\@path_to_perm, $path) <= $perm_index) {
+                     croak( "Different permuted subobjects occur as sources in the same rule" );
+                  }
+               }
+               @path_to_perm=@$path[0..$perm_index];
+               if (grep { $_->flags & $Property::is_permutation } @$path[$perm_index+1..$#$path]) {
+                  croak( "Nested permuted subobjects are not supported" );
+               }
             }
-            $input_perm //= $perms_in_path[0];
          } else {
-            $input_perm=$input->[Property::find_first_in_path($input, $Property::is_permutation)];
+            @path_to_perm = @{$input->[0]}[0..Property::find_first_in_path($input->[0], $Property::is_permutation)];
             last;
          }
       }
    }
 
-   if (defined $input_perm) {
-      if ($input_perm->analyze_rule($self)) {
+   if (@path_to_perm) {
+      if ($path_to_perm[-1]->analyze_rule($self)) {
          $self->flags |= $is_perm_restoring;
          return;
       }
@@ -421,31 +482,30 @@ sub needs_finalization {
 }
 ####################################################################################
 sub finalize {
-   my ($self)=@_;
+   my ($self, $for_twin_rule)=@_;
    my $proto=$self->defined_for;
-   my $need_invalidate=keys %{$proto->all_producers};
    my $perm_deputy=$self->with_permutation;
-   my $permutation=$perm_deputy && $perm_deputy->permutation;
+   my $perm_path=$perm_deputy && $perm_deputy->perm_path;
 
    if ($self->flags & $is_initial) {
       if (defined $perm_deputy) {
          die "rule ", $self->header, " can't trigger permutations at ", $self->source_location, "\n";
       }
-      push @{$proto->producers->{ObjectType::init_pseudo_prop()->key}}, $self;
-      $proto->invalidate_prod_cache(ObjectType::init_pseudo_prop()->key) if $need_invalidate;
+      $proto->add_producers_of(ObjectType::init_pseudo_prop()->key, $self);
 
-      foreach my $input (grep { @$_ > 1 } @{$self->input}) {
-         (undef, my @ancestors)=reverse(@$input);
-         my $prod_key=ObjectType::init_pseudo_prop()->get_prod_key(@ancestors);
-         push @{$proto->producers->{$prod_key}}, $self;
-         $proto->invalidate_prod_cache($prod_key) if $need_invalidate;
+      foreach my $input (@{$self->input}) {
+         foreach my $input_path (grep { @$_ > 1 } @$input) {
+            (undef, my @ancestors)=reverse(@$input_path);
+            my $prod_key=ObjectType::init_pseudo_prop()->get_prod_key(@ancestors);
+            $proto->add_producers_of($prod_key, $self);
+         }
       }
 
       if (@{$self->output}) {
          # an initial rule producing some new properties is only applicable if none of those exists
          my $non_existence_checker=sub {
             # fail on any existing or undefined property
-            not grep { !is_boolean_false($_[0]->lookup_request($_, 1)) } @{$self->output}
+            not grep { !is_boolean_false($_[0]->lookup_descending_path($_, 1)) } @{$self->output}
          };
          my $precond=special Rule(":", $non_existence_checker, $self->defined_for);
          $precond->header="precondition: " . join(" && ", map { "!exists(" . Property::print_path($_) . ")" } @{$self->output}) . " ( " . $self->header . " )";
@@ -455,40 +515,42 @@ sub finalize {
       return;
    }
 
-   my %created_new_multi;
+   my (%created_new_multi, %twin_targets);
    my $i=0;
    foreach my $output (@{$self->output}) {
       my ($created_prop, @prod);
       if (@$output==1) {
          $created_prop=$output->[0];
-         if (defined $perm_deputy) {
-            if (defined (my $prod_list=$permutation->sensitive_props->{$created_prop->key})) {
-               my $action=new PermAction($self, $i, $prod_list);
+         if ($perm_path && @$perm_path==1) {
+            if (defined (my $prod_list=$perm_path->[0]->sensitive_props->{$created_prop->key})) {
+               my $action=new PermAction($self, $i, $prod_list, 0);
                push @{$perm_deputy->actions}, $action;
                push @prod, $action if @$prod_list;
             } else {
                push @prod, $perm_deputy;
             }
          }
-         push @{$proto->producers->{$created_prop->key}}, $self, @prod;
-         $proto->invalidate_prod_cache($created_prop->key) if $need_invalidate;
+         $proto->add_producers_of($created_prop->key, $self, @prod);
 
       } else {
          my ($prop, @ancestors)=reverse(@$output);
          $created_prop=$prop;
-         if (defined $perm_deputy) {
+
+         if ($perm_path && Property::equal_path_prefixes($perm_path, $output)==(my $perm_depth=$#$perm_path)) {
+            my $permutation=$perm_path->[-1];
             my $sub_perm_hash=$permutation->sub_permutations;
+            local_clip_front($output, $perm_depth);
             my $last=@$output-1;
-            my $store_full_path=1;
+            my $sub_perm_exists;
             for (my $depth=1; $depth<=$last; ++$depth) {
                if (defined (my $sub_permutation=$sub_perm_hash->{$output->[$depth-1]->key})) {
                   if (defined (my $prod_list=
                                $depth==$last ? $sub_permutation->sensitive_props->{$output->[$depth]->key}
                                              : $sub_permutation->find_sensitive_sub_property(@$output[$depth..$last]))) {
-                     my $action=new PermAction($self, $i, $prod_list, $depth, $sub_permutation);
+                     my $action=new PermAction($self, $i, $prod_list, $depth+$perm_depth, $sub_permutation);
                      push @{$perm_deputy->actions}, $action;
                      push @prod, $action;
-                     $store_full_path=0;
+                     $sub_perm_exists=1;
                      last;
                   }
                }
@@ -496,9 +558,9 @@ sub finalize {
                defined( $sub_perm_hash=$sub_perm_hash->{$output->[$depth-1]->key} )
                or last;
             }
-            if ($store_full_path) {
+            unless ($sub_perm_exists) {
                if (defined (my $prod_list=$permutation->find_sensitive_sub_property(@$output))) {
-                  my $action=new PermAction($self, $i, $prod_list);
+                  my $action=new PermAction($self, $i, $prod_list, $perm_depth, $perm_depth && $permutation);
                   push @{$perm_deputy->actions}, $action;
                   push @prod, $action;
                } else {
@@ -510,24 +572,31 @@ sub finalize {
          my $flags=get_array_flags($output);
          my $any_mult_depth=$flags >> $any_mult_depth_shift;
          my $common_prefix;  # the longest initial subpath of output occurring among inputs
+         my $bad_twin;
 
          if ($flags & $Property::is_permutation) {
             # rules dealing with permutations may not create new subobjects: pretend a full match
             $common_prefix=$#$output;
          } else {
+            if (!$for_twin_rule && $ancestors[-1]->flags & $Property::is_twin) {
+               $twin_targets{$ancestors[-1]}=1;
+               $bad_twin=1;
+            }
             if ($any_mult_depth==0 || $Application::plausibility_checks) {
                $common_prefix=0;
                foreach my $rule ($self, @{$self->preconditions}) {
                   foreach my $input (@{$rule->input}) {
-                     unless (is_object($input)) {
-                        my $l=equal_list_prefixes($output, $input);
+                     foreach my $input_path (@$input) {
+                        my $l=Property::equal_path_prefixes($output, $input_path);
                         assign_max($common_prefix, $l);
                         if ($Application::plausibility_checks) {
+                           $bad_twin &&= $l>0 if $rule==$self;
+
                            if ($flags & $Property::is_multiple_new &&
                                $l < $#$output &&
-                               $l < $#$input &&
+                               $l < $#$input_path &&
                                $output->[$l]->flags & $Property::is_multiple_new &&
-                               $output->[$l]->property == $input->[$l]) {
+                               $output->[$l]->property == $input_path->[$l]) {
                               die "Multiple subobject ", $output->[$l]->name, " can't be created because it occurs among ",
                                   $rule==$self ? "rule" : "precondition", " sources at ", $self->source_location, "\n";
                            }
@@ -540,6 +609,10 @@ sub finalize {
                      }
                   }
                }
+               if ($Application::plausibility_checks && $bad_twin) {
+                  die "Twin property ", $ancestors[-1]->name, " occurs in all sources and in at least one target - rule should be flipped at ",
+                      $self->source_location, "\n";
+               }
             }
             if ($flags & $Property::is_multiple_new) {
                # strip off all properties below the first new instance,
@@ -553,7 +626,8 @@ sub finalize {
                   foreach ($common_prefix..($any_mult_depth || @$output)-2) {
                      if (($output->[$_]->flags & ($Property::is_multiple | $Property::is_multiple_new)) == $Property::is_multiple) {
                         die "Multiple subobject ", $output->[$_]->name, " in path ", Property::print_path($output),
-                            " must be attributed with `any' or `new' because it does not occur among rule or precondition sources at ", $self->source_location, "\n";
+                            " must be attributed with `any' or `new' because it does not occur among rule or precondition sources at ",
+                            $self->source_location, "\n";
                      }
                   }
                }
@@ -572,8 +646,7 @@ sub finalize {
          for (;;) {
             my $prod_key=$prop->get_prod_key(@ancestors);
             unless ($flags & $Property::is_multiple_new && $created_new_multi{$prod_key}++) {
-               push @{$proto->producers->{$prod_key}}, $self, @prod;
-               $proto->invalidate_prod_cache($prod_key) if $need_invalidate;
+               $proto->add_producers_of($prod_key, $self, @prod);
             }
             last if @ancestors <= $common_prefix;
             $prop=shift @ancestors;
@@ -588,12 +661,12 @@ sub finalize {
    if ($Application::plausibility_checks) {
       if (defined($perm_deputy) && !@{$perm_deputy->actions}) {
          die "None of the target properties is sensitive to the permutation ",
-             $perm_deputy->permutation->name, " which pretends to be triggered at ", $self->source_location, "\n";
+             Property::print_path($perm_deputy->perm_path), " which pretends to be triggered at ", $self->source_location, "\n";
       }
       if ($self->flags & $is_restricted_spez && defined($self->defined_for->preconditions)) {
          foreach my $output (@{$self->output}) {
             foreach my $precond (@{$self->defined_for->preconditions}) {
-               if (defined $precond->matching_input($output)) {
+               if ($precond->has_matching_input($output)) {
                   die "Target ", Property::print_path($output), " is an input to a specialization ", $precond->header, "\n",
                       (@{$self->preconditions} ? "one of preconditions of the production rule must be declared as `override'"
                                                : "the production rule must be equipped with an `override' precondition"),
@@ -613,13 +686,19 @@ sub finalize {
       $proto->override_rule(@$_) for @{$self->overridden_in};
       undef $self->overridden_in;
    }
+
+   if (!$for_twin_rule && keys %twin_targets) {
+      foreach my $twin_prop (keys %twin_targets) {
+         if (defined (my $flipped_rule=flip_rule_for_twin($self, $twin_prop))) {
+            push @{$proto->production_rules}, $flipped_rule;
+            finalize($flipped_rule, 1);
+         }
+      }
+   }
 }
 
-sub checking_precondition {
-   my ($self)=@_;
-   my $permutation=$self->with_permutation->permutation;
-   $permutation->sensitivity_check //= new SensitivityCheck($permutation);
-}
+sub sensitivity_check { $_[0]->with_permutation->perm_path }
+
 ####################################################################################
 # protected:
 sub prepare_header_search_pattern {
@@ -629,7 +708,7 @@ sub prepare_header_search_pattern {
    $_[0] =~ s/(?<=\w)(?=[|:,])/\\s*/g;
    $_[0] =~ s/(?<=[|:,])(?=\w)/\\s*/g;
    # protect characters having special meaning
-   $_[0] =~ s/([.|])/\\$1/g;
+   $_[0] =~ s/([.|()])/\\$1/g;
 }
 
 # protected:
@@ -640,31 +719,30 @@ sub header_search_pattern {
    $_[0] = qr/^(?: $hier_ids_re \s*:\s* )? (?-x:$_[0]) $/x;
 }
 ####################################################################################
-# protected:
+# private:
 # create a closure working as a filter in Object::copy
 sub create_splitting_filter {
-   my ($rule, $perm_dst, $permutation, $descend_path, $prop_hash, $sub_perm_hash, $depth)=@_;
+   my ($rule, $permutation, $descend_path, $prop_hash, $sub_perm_hash, $depth)=@_;
    sub {
-      my ($dst, $pv)=@_;
-      my $prop=$pv->property;
+      my ($dst, $pv, $prop)=@_;
+      $prop //= $pv->property;
       if (defined ($prop_hash) &&
           defined (my $prod_list=$prop_hash->{$prop->key})) {
          my $out=[ @$descend_path, $prop ];
          push @{$rule->output}, $out;
-         my $action=new PermAction($rule, $#{$rule->output}, $prod_list,
-                                   $depth ? ($depth, $permutation) : ());
+         my $action=new PermAction($rule, $#{$rule->output}, $prod_list, $depth, $depth && $permutation);
          push @{$rule->with_permutation->actions}, $action;
-         $perm_dst ||= $dst->add_perm_in_parent($#$descend_path-$depth+1, $permutation);
+         my $perm_dst=$dst->add_perm_in_parent($#$descend_path-$depth+1, $permutation, $rule);
          local_clip_front($out, $depth) if $depth;
          my ($perm_obj)=$perm_dst->descend_and_create($out);
-         return ($perm_obj, $pv->copy($perm_obj));
+         return ($perm_obj, $pv->copy($perm_obj, $prop));
       }
 
       if (defined ($sub_perm_hash) &&
           defined (my $sub_permutation=$sub_perm_hash->{$prop->key})) {
-         return ($dst, $pv->copy($dst, create_splitting_filter($rule, undef, $sub_permutation, [ @$descend_path, $prop ],
-                                                               $sub_permutation->sensitive_props, $sub_permutation->sub_permutations,
-                                                               $#$descend_path+2)));
+         return ($dst, $pv->copy($dst, $prop, create_splitting_filter($rule, $sub_permutation, [ @$descend_path, $prop ],
+                                                                      $sub_permutation->sensitive_props, $sub_permutation->sub_permutations,
+                                                                      $#$descend_path+2)));
       }
 
       my $down_prop=$prop_hash->{$Permutation::sub_key};
@@ -672,23 +750,30 @@ sub create_splitting_filter {
       my $down_sub=$sub_perm_hash->{$Permutation::sub_key};
       $down_sub &&= $down_sub->{$prop->key};
       if (defined($down_prop) || defined($down_sub)) {
-         return ($dst, $pv->copy($dst, create_splitting_filter($rule, $perm_dst, $permutation, [ @$descend_path, $prop ],
-                                                               $down_prop, $down_sub, $depth)));
+         return ($dst, $pv->copy($dst, $prop, create_splitting_filter($rule, $permutation, [ @$descend_path, $prop ],
+                                                                      $down_prop, $down_sub, $depth)));
       }
 
-      return ($dst, $pv->copy($dst));
+      return ($dst, $pv->copy($dst, $prop));
    }
 }
 
 sub permuting {
-   my ($pkg, $proto, $permutation, @descend_to_permuted)=@_;
+   my ($pkg, $proto, $perm_path)=@_;
    my $self=_new($pkg, 'copying', undef, $proto);
    $self->weight=$zero_weight;
-   if (@descend_to_permuted) {
-      $permutation=$proto->create_permutation_for_ancestor($permutation, @descend_to_permuted);
+   $self->with_permutation=new FakeCreatingPermutation($self, $perm_path);
+   my $permutation=local_pop($perm_path);
+   my $filter;
+   if (@$perm_path) {
+      my ($hash, $prop)=Permutation::descend_and_create(my $sub_perms={ }, @$perm_path);
+      $hash->{$prop->key}=$permutation;
+      $hash->{$Permutation::sub_key}->{$prop->key}=$permutation->sub_permutations;
+      $filter=create_splitting_filter($self, undef, [ ], undef, $sub_perms, 0);
+   } else {
+      $filter=create_splitting_filter($self, $permutation, [ ], $permutation->sensitive_props, $permutation->sub_permutations, 0);
    }
-   $self->with_permutation=new FakeCreatingPermutation($self, $permutation);
-   ($self, create_splitting_filter($self, undef, $permutation, [ ], $permutation->sensitive_props, $permutation->sub_permutations, 0))
+   ($self, $filter)
 }
 
 ####################################################################################
@@ -724,10 +809,12 @@ my $sa_INT_save=new POSIX::SigAction('IGNORE');
 # Execute the rule on a separate transaction level
 sub execute {
    my ($self, $object, $force)=@_;
-   my $trans=new RuleTransaction($object, $self);
-   unless ($force || ($self->flags & $is_precondition) || defined($trans->changed)) {
-      $trans->rollback($object);
-      return $exec_OK;
+   unless ($self->flags & $Rule::is_precondition) {
+      my $trans=new RuleTransaction($object, $self);
+      unless ($force || defined($trans->changed)) {
+         $trans->rollback($object);
+         return $exec_OK;
+      }
    }
    dbg_print("applying rule ", $self->header) if $Verbose::rules>2;
    &_execute;
@@ -742,7 +829,6 @@ sub _execute {
    eval {
       sigaction SIGINT, $sa_break, $sa_INT_save;
       sigaction SIGALRM, $sa_break;
-      sigaction SIGPIPE, $sa_break;
       my $alarm_time=$timeout && !($self->flags & $is_precondition);
       alarm $alarm_time if $alarm_time;
       if (wantarray || ($self->flags & $is_precondition)) {
@@ -766,12 +852,11 @@ sub _execute {
          # ignore the return code of the production rule
          $rc=$exec_OK;
       }
-      $object->commit;
+      $object->commit unless ($self->flags & $Rule::is_precondition);
       $died=0;
    };
    sigaction SIGINT, $sa_INT_save;
    $SIG{ALRM}='IGNORE';
-   $SIG{PIPE}='DEFAULT';
 
    if ($died) {
       $rc=$exec_failed;
@@ -783,25 +868,22 @@ sub _execute {
       }
       $object->failed_rules->{$self}=1;
       $object->failed_rules->{$self->with_permutation}=1 if defined $self->with_permutation;
-      $object->rollback;
+      $object->rollback unless ($self->flags & $Rule::is_precondition);
    }
    wantarray ? ($rc, $retval) : $rc;
 }
 ####################################################################################
 sub list_results {
-   my $self=shift;
+   my ($self, $proto)=@_;
    $self->flags & ($is_function | $is_precondition)
      ||
    scalar(grep { get_array_flags($_) & $Property::is_permutation } @{$self->input})
    ? ()
    : map {
-        my $proto=$_[0];
-        my $i=$#$_;
+        my $obj_type=$proto;
         join(".", map {
-           my $name=$proto->property_name($_);
-           if ($i--) {
-              $proto=$_->specialization($proto)->type;
-           }
+           my $name=$obj_type->property_name($_);
+           $obj_type=$_->type;
            $name
         } @$_)
      } @{$self->output}
@@ -837,7 +919,7 @@ sub descend_to_source {
    my ($self, $object)=@_;
    my ($prop, $content_index);
    if (@{$self->input}) {
-      if (($object, $prop)=$object->descend($self->input->[0])
+      if (($object, $prop)=$object->descend($self->input->[0]->[0])
           and defined ($content_index=$object->dictionary->{$prop->key})) {
          $object->contents->[$content_index]
       } else {
@@ -884,14 +966,14 @@ use Polymake::Struct (
 sub execute { $exec_OK }
 
 ####################################################################################
-package __::Deputy;
+package Polymake::Core::Rule::Deputy;
 
 # Base class for various rule proxies, especially those used in the Scheduler.
 use Polymake::Struct (
    [ new => '$' ],
    [ '$rule' => '#1' ],     # the proper Rule doing the hard job
-   [ '$flags' => '#1 ->flags' ],
-   [ '$weight' => '#1 ->weight' ],
+   [ '$flags' => '#1->flags' ],
+   [ '$weight' => '#1->weight' ],
 );
 
 sub header { $_[0]->rule->header }
@@ -904,21 +986,23 @@ sub dyn_weight { $_[0]->rule->dyn_weight }
 sub overridden_in { $_[0]->rule->overridden_in }
 sub with_permutation { $_[0]->rule->with_permutation }
 sub defined_for { $_[0]->rule->defined_for }
-sub matching_input { $_[0]->rule->matching_input($_[1]) }
+sub has_matching_input { $_[0]->rule->has_matching_input($_[1]) }
+sub not_for_twin { $_[0]->rule->not_for_twin }
+
 ####################################################################################
 package __::CreatingPermutation;
 
 use Polymake::Struct (
    [ '@ISA' => 'Deputy' ],
    [ new => '$$' ],
-   [ '$rule' => 'weak( #1 )' ], # production Rule
-   [ '$permutation' => '#2' ],  # permutation Property
+   [ '$rule' => 'weak(#1)' ], # production Rule
+   [ '$perm_path' => '#2' ],    # property path leading to a permutation
    '@actions',                  # PermAction descriptors
 );
 
 sub header {
    my ($self)=@_;
-   $self->rule->header . " ( creating " . $self->permutation->name . " )";
+   $self->rule->header . " ( creating " . Property::print_path($self->perm_path) . " )";
 }
 
 sub with_permutation { undef }
@@ -933,7 +1017,8 @@ sub execute {
 
 sub record_sensitivity {
    my ($self, $object)=@_;
-   $object->sensitive_to->{$self->permutation->key}=1;
+   my ($sub_obj, $perm)=$object->descend($self->perm_path);
+   $sub_obj->sensitive_to->{$perm->key}=1;
    foreach my $action (@{$self->actions}) {
       if ($action->depth) {
          my $out=$self->rule->output->[$action->output];
@@ -958,8 +1043,8 @@ sub execute {
 package __::PermAction;
 
 use Polymake::Struct (
-   [ new => '$$$;$$' ],
-   [ '$perm_trigger' => 'weak( #1 )' ], # production rule triggering the permutation
+   [ new => '$$$$;$' ],
+   [ '$perm_trigger' => 'weak(#1)' ], # production rule triggering the permutation
    [ '$output' => '#2' ],               # index into its output where the sensitive property lies
    [ '$producers' => '#3' ],            # ( Rule ) moving the output property back to the basis
    [ '$depth' => '#4' ],                # how deep to descend in the base object
@@ -969,10 +1054,10 @@ use Polymake::Struct (
 sub header {
    my $self=shift;
    Property::print_path($self->perm_trigger->output->[$self->output])
-   . " extracted from " . $self->permutation->name . " after " . $self->perm_trigger->header;
+   . " extracted from " . Property::print_path($self->perm_path) . " after " . $self->perm_trigger->header;
 }
 sub list_results { () }
-sub permutation { $_[0]->perm_trigger->with_permutation->permutation }
+sub perm_path { $_[0]->perm_trigger->with_permutation->perm_path }
 sub flags { $is_perm_action }
 sub weight { $zero_weight }
 sub enabled { @{$_[0]->producers}!=0 }
@@ -980,16 +1065,12 @@ sub enabled { @{$_[0]->producers}!=0 }
 sub preconditions { [ ] }
 sub with_permutation { undef }
 sub overridden_in { undef }
+sub not_for_twin { $_[0]->perm_trigger->not_for_twin }
 
 ####################################################################################
-package __::SensitivityCheck;
+package __::PermutationPath;
 
-use Polymake::Struct (
-  [ new => '$' ],
-  [ '$permutation' => 'weak( #1 )' ],
-);
-
-sub header { "sensitivity check for " . (shift)->permutation->name }
+sub header { "sensitivity check for " . &Property::print_path }
 sub list_results { () }
 sub flags { $is_function }
 sub weight { $zero_weight }
@@ -999,7 +1080,17 @@ sub with_permutation { undef }
 sub execute {
    my ($self, $object)=@_;
    dbg_print("performing ", $self->header) if $Verbose::rules>2;
-   $object->has_sensitive_to($self->permutation) ? $exec_failed : $exec_OK
+   my ($sub_obj, $perm)=$object->descend($self);
+   $sub_obj->has_sensitive_to($perm) ? $exec_failed : $exec_OK
+}
+
+sub find_sensitive_sub_property {
+   my ($self, @path)=@_;
+   if (Property::equal_path_prefixes($self, \@path)==$#$self) {
+      $self->[-1]->find_sensitive_sub_property(splice @path, $#$self)
+   } else {
+      undef
+   }
 }
 
 ####################################################################################
@@ -1016,14 +1107,14 @@ use Polymake::Struct (
 declare $hide=3;   # suppresses display if assigned to ->shown
 
 sub display {
-   my $self=shift;
+   my ($self)=@_;
    dbg_print( "used package ", $self->product, "\n", $self->text, "\n" );
    $self->shown=1;
 }
 
 sub toFileString {
-   my $self=shift;
-   $self->file_string ||= do {
+   my ($self)=@_;
+   $self->file_string //= do {
       my ($copyright)= $self->text =~ /(copyright\b.*)/im;
       my ($URL)= $self->text =~ /(?:url:?\s+)(\S+)/im;
       if (!defined($URL) && $self->text =~ m{(?:\w+://|(?:www|ftp)\.)\S+}) {
